	SUBROUTINE OPENFITS(LU,IOFL,FILEKEYWORD,FILENAME,MAPKEY,PLANEKEY,
	1NMAPS,NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
C
C       This subroutine opens a FITS image file with data formats of BYTE,
C  INTEGER*2, INTEGER*4, REAL*4, REAL*8, COMPLEX*8 (real and imaginary parts
C  are each REAL*4) and COMPLEX*16 (real and imaginary parts are each REAL*8).
C  For existing files where no new disk storage is required, a call to OPENFITS
C  will open a FITS file of any data type for reading or writing, and place 
C  pertinent information about the file in the FITSFACTS COMMON blocks.
C  For existing files where the number of image extensions (maps) is to be
C  increased, a call to OPENFITS will create new storage for one or more maps
C  of any single size and any single data type, and write the necessary
C  keywords into a new header record.  If the old file had only one map, a
C  a principal header plus a header for each map will be created.  For new
C  files, a call to OPENFITS will create one or more maps of any single size
C  and any single data type, and write the necessary keywords into the
C  headers.  One can choose between specifying the size and data type of the
C  new file in the call, or having OPENFITS prompt the user for the
C  informaiton.
C  
C			Donald H. Gudehus, June 27, 2005
C			Version 2, D. Gudehus, September 2, 2006
C			Introduced PRGFMAT_IN to avoid buffer overflow
C			Version 3, D. Gudehus, January 6, 2008
C			Added code for QUIET.  Reversed NCOLS and NROWS
C			in argument list.  Made FILENAME a returned value.
C			Version 4, D. Gudehus, February 19, 2008
C			Ensured that the number of rows and columns would be
C			returned when an old file was opened as a new file.
C
C	LU	    (C)	   Logical unit for I/O
C	IOFL	    (C/R)  Flag determining whether the file is to be opened
C			   for reading or updating pixel values (IOFL =
C			   .TRUE.) or instead creating a new file, adding maps
C			   to an existing file, or updating pixel values (IOFL
C			   = .FALSE.).  If IOFL is .TRUE., the file must exist
C			   to not cause an error.  If IOFL is .FALSE., the
C			   file may or may not exist.  In practice, IOFL =
C			   .TRUE. is used for an input file, and IOFL =
C			   .FALSE. is used for an output file.  For IOFL =
C			   .TRUE. and the file exists, the default in the
C			   prompt for the map number is 1, whereas for IOFL
C			   = .FALSE. and the file exists, the default in
C			   the prompt for the map number is 1 after the last
C			   map number.  If IOFL is .FALSE. in the call and the
C			   file exists, IOFL is returned as .TRUE.
C	FILEKEYWORD (C)	   Keyword to be used in identifying the file in the
C			   call to USRINP
C	FILENAME    (C/R)  The name of the file;  if it is blank, it is
C		 	   prompted for.
C	MAPKEY	    (C)	   The key for whether OPENFITS supplies the prompt
C			   for the desired map number (0) or whether the
C			   calling routine does (>=1).  If MAPKEY >=1, in
C			   an existing file, the desired map number is set
C			   equal to MAPKEY, i.e., IMNUM in the common block
C			   MAPN is set to MAPKEY.  When adding maps to an
C			   existing file (IOFL=.FALSE.), maps will be added
C			   so as to reach a number equal to MAPKEY.  For a
C			   new file, MAPKEY is only used if more than one
C			   map was created.  If the calling program will
C			   loop through several maps, MAPKEY should be 0.
C	PLANEKEY    (C)    The key for whether OPENFITS supplies prompt for
C			   the desired plane number (0) or whether calling
C			   routine does (>=1).  If PLANEKEY >=1, then the
C			   desired plane number is set equal to PLANEKEY,
C			   i.e., IPNUM in the common block PLANEN is set to
C			   PLANEKEY.
C	NMAPS	    (C/R)  For an existing or new file, NMAPS is returned as
C			   the total number of maps in the file.  For a new
C			   file, if NMAPS is 0 in the call, the number of 
C			   maps to create will be asked for;  if > 0, the
C			   number of maps to be created will be set to NMAPS.
C	NPLNS	    (C/R)  For an existing or new file, NPLNS is returned as
C			   the total number of planes in the chosen map.  For a
C			   new map or file, if NPLNS is 0, the number of planes to
C			   create will be asked for;  if > 0, the number of
C			   planes to be created will be set to NPLNS.  For
C			   complex data, NPLNS will be set to 2.
C	NCOLS	    (C/R)  For an existing or new file, NCOLS is retuned as
C			   the number of columns per plane in the chosen map.
C			   For a new file, or for a new map in an existing
C			   file, if NCOLS is 0, the number of columns will be
C			   asked for;  if > 0, the number of columns will be
C			   set to NCOLS.
C	NROWS	    (C/R)  For an existing or new file, NROWS is returned as
C			   the number of rows per plane in the chosen map.  For
C			   a new file, or for a new map in an existing file,
C			   if NROWS is 0, the number of rows will be asked
C			   for;  if > 0, the number of rows will be set to
C			   NROWS.
C	PRGFMAT     (C/R)  For an existing or new file, PRGFMAT is returned
C			   as the data format in the file.  The possible values
C			   are I1, I2, I4, R4, R8, C8, and C16.  For a new file
C			   or for a new map in an existing file, if PRGFMAT is
C			   blank, the value will be asked for.
C	STATUS      (R)    A status return.  If there are no errors, the
C			   returned value is 0;  if IOFL is .TRUE. in the call
C			   and the file does not exist a 1 is returned;  if
C			   the file is already open, a 1 is returned;  if
C			   there are errors in reading the file, a 2 is
C			   returned.
C
	CHARACTER*(*) FILEKEYWORD
	CHARACTER*(*) FILENAME
	CHARACTER*3 PRGFMAT
	INTEGER PLANEKEY,NPLNS,STATUS
	LOGICAL IOFL
C
	INCLUDE 'fitsfacts.for'
C
	BYTE PHYSREC(3000),PHYSREC_2(3000)
	CHARACTER*3000 CHARPHYSREC,CHARPHYSREC_2
	EQUIVALENCE (PHYSREC,CHARPHYSREC),(PHYSREC_2,CHARPHYSREC_2)
	CHARACTER*3 DATFMT
	CHARACTER*10 CHFITSMHV
	CHARACTER*37 MESS1
	CHARACTER*38 MESS2
	CHARACTER*58 MESS3
	CHARACTER*80 NAME,KEYCARD,KEYCARD_PREVIOUS,KEYCARD_TEMP,
	1PRGFMAT_IN
	LOGICAL FILE_EXISTS,FILE_IS_OPEN,DONE,INCONSISTENT
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	CHARACTER*80 DNAME
	BYTE B_DNAME(80)
	EQUIVALENCE (B_DNAME(1),DNAME(1:1))
	LOGICAL EXTENDED
	COMMON /DFILE/ DNAME,EXTENDED
C
	IF (LU.LT.LUMIN.OR.LU.GT.LUMAX)THEN
	  CALL ERROR(-4,'LU outof range in OPENFITS')
	END IF
	STATUS=0
	IBLKLEN=2880
C
	IF (.NOT.IOFL) GO TO 500
C
C  Section for IOFL=.TRUE. (Here we are not creating a new file or new
C  maps, but just reading or updating values in a map) **************
C
	IF (FILENAME.EQ.' ') THEN
C  Default filename is contained in 'DNAME' if a file has been opened
C  this session;  if not in DNAME, then one is generated using the LU number.
C  MIIPS.FOR initializes DNAME, however user applications cannot be
C  relied upon to initialize DNAME.  In that case, DNAME is not equal to
C  blanks, but rather zeroes.  Therefore, B_DNAME is checked here also.
	  IF (DNAME.EQ.' '.OR.B_DNAME(1).EQ.0) THEN
	    ENCODE(12,105,NAME) LU !  Default filename
 105	    FORMAT('fits0'I2,'.fits')
	  ELSE
	    NAME=DNAME !  NAME is set equal to the last file
		       !  opened
	  END IF
	  IE=INDEX(FILEKEYWORD,'=')
	  CALL USRINP(%REF(NAME),1,'A',80,1,FILEKEYWORD(1:IE),
	1 'Enter the filename:  (<CR> for default)')
	  CALL ALPHA(%REF(NAME),1,80,NCHAR,NFULL,NREST,2)
	  FITSFILENAME(LU)=NAME
	  FILENAME=NAME !  Returns the name for convenience
	ELSE
	  NAME=FILENAME !  NAME is set equal to the file supplied
			!  in the call to this subroutine
	  FITSFILENAME(LU)=FILENAME
	END IF
	DNAME=NAME !  Updates default file name
C
	INQUIRE(FILE=NAME,EXIST=FILE_EXISTS,OPENED=FILE_IS_OPEN)
	IF (.NOT.FILE_EXISTS) THEN
	  CALL ERROR(-3,'The chosen file does not exist')
	  STATUS=1
	  RETURN
	END IF
	IF (FILE_IS_OPEN) THEN
	  CALL ERROR(-3,'The file is already open')
	  STATUS=1
	  RETURN
	END IF
C
	OPEN(LU,STATUS='OLD',ACCESS='DIRECT',RECL=2880,
	1FORM='UNFORMATTED',FILE=NAME)
C
	CALL GET_FITS_INFO(LU)
	NMAPS=NUMMAPS(LU)
C  Obtain the map number from the user.
	IMNUM=1
	IF (NMAPS.EQ.1) THEN
	  IF (.NOT.QUIET(LU)) CALL MESSAGE('There is one map in the file')
	ELSE IF (NMAPS.GT.1) THEN
	  ENCODE(37,1215,MESS1) NMAPS
1215	  FORMAT('There are'I4,' maps in the file'7X)
	  IF (.NOT.QUIET(LU)) CALL MESSAGE(MESS1)
	  IF (MAPKEY.EQ.0) THEN
	    CALL USRINP(IMNUM,1,'I',4,1,'MAP=',
	1   'Enter the map number:  (default = 1)')
	  ELSE
	    IMNUM=MAPKEY
	  END IF
	ELSE
	  CALL ERROR(-3,'Error in obtaining number of maps from file')
	  STATUS=2
	END IF
	IF (IMNUM.GT.NMAPS) THEN
	  CALL ERROR(-1,'Requested map does not exist')
	  IMNUM=1
	  RETURN
	END IF
C
	NPLNS=NAXIS3(IMNUM,LU)
	NROWS=NAXIS2(IMNUM,LU)
	NCOLS=NAXIS1(IMNUM,LU)
	DATFMT=CHFITSMHV(LU,IMNUM,'BITPIX','?')
	PRGFMAT=DATFMT
	IF (DATFMT.EQ.'?') THEN
	  CALL ERROR(-3,'Error getting data format')
	  STATUS=2
	END IF
	IF (.NOT.QUIET(LU)) CALL MESSAGE('Reading from the file')
	ENCODE(58,1250,MESS3) NROWS,NCOLS,DATFMT(1:3),IMNUM
1250	FORMAT
	1(I4,' rows of ',I4,' pixels in ',A3,' format for map number ',I4)
	IF (.NOT.QUIET(LU)) CALL MESSAGE(MESS3)
C
C  Obtain the plane number from the user
	IPNUM=1
	IF (NPLNS.EQ.1) THEN
	  IF (.NOT.QUIET(LU)) CALL MESSAGE('There is one plane in this map')
	ELSE IF (NPLNS.GT.1) THEN
	  IF (.NOT.QUIET(LU)) WRITE (6,1255) NPLNS
1255	  FORMAT(' ','There are ',I4,' planes in this map')
	  IF (.NOT.QUIET(LU)) WRITE (7,1255) NPLNS
	  IF (PLANEKEY.EQ.0) THEN
	    IF (NPLNS.EQ.2.AND.COMPLX(IMNUM,LU)) THEN
	      CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1     'Enter the plane number (real = 1, imaginary =2):  '//
	2     '(default = 1)')
	    ELSE IF (NPLNS.NE.2.AND.COMPLX(IMNUM,LU)) THEN
	      CALL ERROR(-2,
	1     'Inconsistent number of planes for COMPLEX data')
	    ELSE
	       CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1     'Enter the plane number:  (default = 1)')
	    END IF
	  ELSE
	    IPNUM=PLANEKEY
	  END IF
	ELSE
	  CALL ERROR(-3,'Error in obtaining number of planes in the map')
	  STATUS=2
	END IF
	RETURN
C
C  Section for IOFL=.FALSE. (opens for writing new file or map(s)) *******
C
 500	CONTINUE
C  Initialize the variable for whether we extended maps or not.  For a
C  new file, we don't extend maps, we create them.  For an old file we
C  may or may not extend the maps.
	EXTENDED=.FALSE.
	IF (FILENAME.EQ.' ') THEN
C  Default filename is contained in 'DNAME' if a file has been opened
C  this session;  if not, then one is generated using the LU number.
	  IF (DNAME.EQ.' '.OR.B_DNAME(1).EQ.0) THEN
	    ENCODE(12,105,NAME) LU !  Default filename
	  ELSE
	    NAME=DNAME !  NAME is set equal to the last file
		       !  opened
	  END IF
	  IE=INDEX(FILEKEYWORD,'=')
	  CALL USRINP(%REF(NAME),1,'A',80,1,FILEKEYWORD(1:IE),
	1 'Enter the filename:  (<CR> for default)')
	  CALL ALPHA(%REF(NAME),1,80,NCHAR,NFULL,NREST,2)
	  FITSFILENAME(LU)=NAME
	  FILENAME=NAME
	ELSE
	  NAME=FILENAME !  NAME is set equal to the file supplied
			!  in the call to this subroutine
	  FITSFILENAME(LU)=FILENAME
	END IF
	DNAME=NAME !  Updates default file name
	INQUIRE(FILE=NAME,EXIST=FILE_EXISTS,OPENED=FILE_IS_OPEN)
	IF (FILE_IS_OPEN) THEN
	  CALL ERROR(-3,'The file is already open')
	  STATUS=1
	  RETURN
	END IF
C
	IF (FILE_EXISTS) THEN
	  GO TO 700
	END IF
C
	CALL MESSAGE('Creating new file')
	OPEN(LU,STATUS='NEW',ACCESS='DIRECT',RECL=2880,FORM='UNFORMATTED',
	1FILE=NAME,IOSTAT=ISTAT)
	IF (ISTAT.NE.0) THEN
	  CALL ERROR(-3,'Occurred while opening file')
	  STATUS=2
	END IF
C  Get some information from the user if it is not present in the argument
C  list.
	IF (NMAPS.EQ.0) THEN
	  NMAPS=1
	  CALL USRINP(NMAPS,1,'I',4,1,'NUMBER_OF_MAPS=',
	1 'Enter the number of maps desired in the new file:  (default = 1)')
	END IF
	IF (NCOLS.EQ.0) THEN
	  CALL USRINP(NCOLS,1,'I',4,0,'NUMBER_OF_COLUMNS=',
	1 'Enter the number of columns for the new file:')
	END IF
	IF (NROWS.EQ.0) THEN
	  CALL USRINP(NROWS,1,'I',4,0,'NUMBER_OF_ROWS=',
	1 'Enter the number of rows for the new file:')
	END IF
C  Determine how many records to add
	IF (NMAPS.EQ.1) THEN
	  INCREMENT=0
	ELSE IF (NMAPS.GT.1) THEN
	  INCREMENT=1 !  Includes a primary header
	END IF
	IF (PRGFMAT.EQ.' ') THEN
	  PRGFMAT_IN=' ' !  Longer word here to avoid buffer overflow
	  DO WHILE(PRGFMAT_IN.NE.'I1'.AND.PRGFMAT_IN.NE.'I2'.AND.
	1 PRGFMAT_IN.NE.'I4'.AND.PRGFMAT_IN.NE.'R4'.AND.PRGFMAT_IN.NE.
	2 'R8'.AND.PRGFMAT_IN.NE.'C8'.AND.PRGFMAT_IN.NE.'C16')
	    PRGFMAT_IN='R4'
	    CALL MESSAGE('Enter the data format to use:')
	    CALL MESSAGE('     I1 = Unsigned 8-bit integer')
	    CALL MESSAGE('     I2 = 16-bit twos-complement binary integer')
	    CALL MESSAGE('     I4 = 32-bit twos-complement binary integer')
	    CALL MESSAGE('     R4 = IEEE single precision floating point'//
	1   ' (default)')
	    CALL MESSAGE('     R8 = IEEE double precision floating point')
	    CALL MESSAGE('     C8 = Complex - real and imaginary are R4')
	    CALL MESSAGE('    C16 = Complex - real and imaginary are R8')
	    CALL USRINP(%REF(PRGFMAT_IN),1,'A',80,1,'DATA_FORMAT=',' ')
	    CALL ALPHA(%REF(PRGFMAT_IN),1,80,NC,NF,NR,2)
	  END DO
	  PRGFMAT=PRGFMAT_IN
	END IF
	IF (NPLNS.EQ.0) THEN !  Key for prompt
	  IF (PRGFMAT.EQ.'C8'.OR.PRGFMAT.EQ.'C16') THEN
	    INPLNS=2
	    CALL MESSAGE('The map(s) will contain two planes for the real '//
	1   'and imaginary parts')
	  ELSE
	    INPLNS=1
	    CALL USRINP(INPLNS,1,'I',4,1,'NUMBER_OF_PLANES=',
	1   'Enter the number of planes for the new file (default = 1):')
	  END IF
	  NPLNS=INPLNS
	END IF
C  Check for consistency if data is  complex
	IF (PRGFMAT.EQ.'C8'.OR.PRGFMAT.EQ.'C16') THEN
	  INCONSISTENT=.FALSE.
	  IF (NPLNS.NE.2) INCONSISTENT=.TRUE.
	  IF (INCONSISTENT) CALL ERROR(-1,'Complex data requires two planes')
	END IF
C
	NPIXELSPERMAP=NPLNS*NROWS*NCOLS
	IF (PRGFMAT.EQ.'I1') THEN
	  NBYTESPERMAP=NPIXELSPERMAP
	ELSE IF (PRGFMAT.EQ.'I2') THEN
	  NBYTESPERMAP=2*NPIXELSPERMAP
	ELSE IF (PRGFMAT.EQ.'I4'.OR.PRGFMAT.EQ.'R4'.OR.PRGFMAT.EQ.'C8') THEN
	  NBYTESPERMAP=4*NPIXELSPERMAP
	ELSE IF (PRGFMAT.EQ.'R8'.OR.PRGFMAT.EQ.'C16') THEN
	  NBYTESPERMAP=8*NPIXELSPERMAP
	END IF
C  Compute the number of records per map
	NRECSPERMAP=(NBYTESPERMAP-1)/IBLKLEN+1 !  Excluding the header
	NRECSTOADD=NRECSPERMAP*NMAPS+(NMAPS+INCREMENT)
C  First fill the file with ASCII nulls, then fill the header(s) with
C  ASCII blanks below
	DO I=1,IBLKLEN
	  PHYSREC(I)=0
	END DO
	DO IR=1,NRECSTOADD
	  WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	  IF (ISTAT.NE.0) THEN
	    CALL ERROR(-3,'Error in writing physical record')
	    STATUS=2
	  END IF
	END DO
C  In order to write new keywords, there has to be some information in the
C  common blocks about the the headers, maps, and records.
	NUMRECORDS(LU)=NRECSTOADD
	DO N=0,NMAPS
	  NUMKEYCARDS(N,LU)=0
	END DO
	NUMMAPS(LU)=NMAPS
C  ASCII blanks now
	DO I=1,IBLKLEN
	  PHYSREC(I)=32
	END DO
C  The start and end records for the primary header, assuming the header
C  is initially just one record.
	HEADERRECORDS(1,0,LU)=1
	HEADERRECORDS(2,0,LU)=1
C  For the map headers
	IF (NMAPS.EQ.1) THEN
C  For only one map, the map header is also the primary header
	  HEADERRECORDS(1,1,LU)=1
	  HEADERRECORDS(2,1,LU)=1
	  WRITE(LU,REC=1,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	  IF (ISTAT.NE.0) THEN
	    CALL ERROR(-1,'Error in writing physical record')
	    STATUS=2
	  END IF
	ELSE IF (NMAPS.GT.1) THEN
C  First do the principal header
	  WRITE(LU,REC=1,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	  DO N=1,NMAPS
	    HEADERRECORDS(1,N,LU)=2+(NRECSPERMAP+1)*(N-1)
	    HEADERRECORDS(2,N,LU)=HEADERRECORDS(1,N,LU)
	    DO IR=HEADERRECORDS(1,N,LU),HEADERRECORDS(2,N,LU)
	      WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	      IF (ISTAT.NE.0) THEN
	        CALL ERROR(-1,'Error in writing physical record')
	        STATUS=2
	      END IF
	    END DO
	  END DO
	END IF
	IF (NMAPS.GT.1) THEN
	  MSTART=0
	ELSE
	  MSTART=1
	END IF
C  Now we are ready to write keycards
	DO M=MSTART,NMAPS
	  IF (NMAPS.GT.1.AND.M.NE.0) THEN
	    CALL WRFITSMHST(LU,M,'XTENSION','''IMAGE   ''','C',
	1   'Image extension',.TRUE.)
	  ELSE
	    CALL WRFITSMHV(LU,M,'SIMPLE',1.0D0,'L','Standard FITS file',.TRUE.)
	  END IF
	  IF (PRGFMAT.EQ.'I1') THEN
	    CALL WRFITSMHV(LU,M,'BITPIX',8.0D0,'I4',' ',.TRUE.)
	  ELSE IF (PRGFMAT.EQ.'I2') THEN
	    CALL WRFITSMHV(LU,M,'BITPIX',16.0D0,'I4',' ',.TRUE.)
	  ELSE IF (PRGFMAT.EQ.'I4') THEN
	    CALL WRFITSMHV(LU,M,'BITPIX',32.0D0,'I4',' ',.TRUE.)
	  ELSE IF (PRGFMAT.EQ.'R4'.OR.PRGFMAT.EQ.'C8') THEN
	    CALL WRFITSMHV(LU,M,'BITPIX',-32.0D0,'I4',' ',.TRUE.)
	  ELSE IF (PRGFMAT.EQ.'R8'.OR.PRGFMAT.EQ.'C16') THEN
	    CALL WRFITSMHV(LU,M,'BITPIX',-64.0D0,'I4',' ',.TRUE.)
	  END IF
	  IF (NMAPS.GT.1.AND.M.EQ.0) THEN
	    CALL WRFITSMHV(LU,M,'NAXIS',0.0D0,'I4',' ',.TRUE.)
	  ELSE
	    CALL WRFITSMHV(LU,M,'NAXIS',3.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS1',NCOLS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS2',NROWS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS3',NPLNS*1.0D0,'I4',' ',.TRUE.)
	  END IF
C  The PCOUNT and GCOUNT keywords are not required in the primary header
C  but they must follow immediately after the NAXIS keywords in the
C  extension headers.
	  IF (NMAPS.GT.1.AND.M.NE.0) THEN
	    CALL WRFITSMHV(LU,M,'PCOUNT',0.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'GCOUNT',1.0D0,'I4',' ',.TRUE.)
	  END IF
	  IF (M.NE.0.AND.(PRGFMAT.EQ.'C8'.OR.PRGFMAT.EQ.'C16'))
	1 CALL WRFITSMHST(LU,M,'CTYPE3','''COMPLEX ''','C',' ',.TRUE.)
	  IF (NMAPS.GT.1.AND.M.EQ.0) THEN
	    CALL WRFITSMHV(LU,M,'EXTEND',1.0D0,'L',' ',.TRUE.)
	  END IF
	  CALL WRFITSMHV(LU,M,'END',0.0D0,'I4',' ',.TRUE.)
	END DO
C
C  Set a default value of 1 for the map number in the COMMON block.
	IMNUM=1
C  Prompt the user for the map number if MAPKEY=0 and more than one map was
C  created.
	IF (MAPKEY.EQ.0.AND.NMAPS.GT.1) THEN
	  CALL USRINP(IMNUM,1,'I',4,1,'MAP=',
	1 'Enter the map number:  (default = 1)')
	END IF
C  Set a default value of 1 for the plane number in the COMMON block.
	IPNUM=1
C  Prompt for the plane number if PLANEKEY=0 and more than one plane was
C  created.
	IF (PLANEKEY.EQ.0.AND.NPLNS.GT.1) THEN
	  CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1 'Enter the plane number:  (default = 1)')
	END IF
C
C  Update the FITS information in the common blocks
	CALL GET_FITS_INFO(LU)
C
	RETURN
C
C
C  Section for IOFL=.FALSE. (opens for writing:  file exists but the 
C  requested map does not exist, or map exists and we want to update
C  values)         **********************************
C
 700	OPEN(LU,STATUS='OLD',ACCESS='DIRECT',RECL=2880,FORM='UNFORMATTED',
	1FILE=NAME) !  Opens an existing file for adding maps or writing
		    !  to an existing map
	CALL MESSAGE('Writing to an existing file')
	IOFL=.TRUE. !  Tells the calling program that the file exists
	CALL GET_FITS_INFO(LU)
	NMAPS=NUMMAPS(LU) !  Existing number from FITS COMMON block
	IF (NMAPS.EQ.1) THEN
	  CALL MESSAGE('There is one map in the file')
	ELSE IF (NMAPS.GT.1) THEN
	  ENCODE(37,1215,MESS1) NMAPS
	  CALL MESSAGE(MESS1)
	END IF
	IF (MAPKEY.EQ.0) THEN !  Key for prompt
	  IMNUM=1+NMAPS
	  CALL USRINP(IMNUM,1,'I',4,1,'MAP=',
	1 'Enter the map number:  (default = one after the last map)')
	ELSE
	  IMNUM=MAPKEY
	END IF
C  Handle the case where the map actually does exist, and then return without
C  writing any new storage.  For future development we could check the size
C  against what is requested before returning.
	IF (IMNUM.GT.0.AND.IMNUM.LE.NMAPS) THEN
	  CALL MESSAGE('Writing to an existing map')
	  NPLNS=NAXIS3(IMNUM,LU)
	  IF (NPLNS.EQ.1) THEN
	    CALL MESSAGE('There is one plane in the map')
	  ELSE IF (NPLNS.GT.1) THEN
	    ENCODE(38,1270,MESS2) NPLNS
1270	    FORMAT('There are'I4,' planes in the map'7X)
	    CALL MESSAGE(MESS2)
	  END IF
	  IF (PLANEKEY.EQ.0) THEN !  Key for prompt
	    IPNUM=1
	    IF (NPLNS.GT.1) THEN
	      CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1     'Enter the plane number:  (default = 1)')
	    END IF
	  ELSE
	    IPNUM=PLANEKEY
	  END IF
	  NCOLS=NAXIS1(IMNUM,LU)
	  NROWS=NAXIS2(IMNUM,LU)
	  RETURN !  No problem
	END IF
C
	IF (IMNUM-NMAPS.EQ.1) THEN
	  WRITE (6,1300) IMNUM-NMAPS
1300	  FORMAT(' ','Creating ',I3,' new map')
	  WRITE (7,1300) IMNUM-NMAPS
	ELSE
	  WRITE (6,1301) IMNUM-NMAPS
1301	  FORMAT(' ','Creating ',I3,' new maps')
	  WRITE (7,1301) IMNUM-NMAPS
	END IF
	EXTENDED=.TRUE.
C
C  The information from the last existing map is used to provide defaults
C  for the size and data type of the next map.
C
C  Possible future development:
C  Three possibilities for MAPKEY and PRGFMAT:  program prompts, (0  or 
C  blank in call), same as previous map (-1 in call), set in call (nonzero
C  or nonblank value)
C
	IROWS=I4FITSMHV(LU,NMAPS,'NAXIS2',0)
	IF (IROWS.EQ.0) THEN
	  CALL ERROR(-3,'Error getting number of rows')
	  STATUS=2
	END IF
	ICOLS=I4FITSMHV(LU,NMAPS,'NAXIS1',0)
	IF (ICOLS.EQ.0) THEN
	  CALL ERROR(-3,'Error getting number of columns')
	  STATUS=2
	END IF
	DATFMT=CHFITSMHV(LU,NMAPS,'BITPIX','?')
	IF (DATFMT.EQ.'?') THEN
	  CALL ERROR(-3,'Error getting data format')
	  STATUS=2
	END IF
C  Only print out the information for the last existing map if the user
C  is to be given a prompt for entering the number of columns, rows, or
C  data format.
	IF (NROWS.EQ.0.OR.NCOLS.EQ.0.OR.PRGFMAT.EQ.' ') THEN
	  CALL MESSAGE('Reading from the last map of the file')
	  ENCODE(38,1350,MESS2) IROWS,ICOLS,DATFMT(1:3)
1350	  FORMAT(I4,' rows of 'I4,' pixels in 'A3,' format')
	  CALL MESSAGE(MESS2)
	  IPLNS=NAXIS3(NMAPS,LU)
	  IF (IPLNS.EQ.1) THEN
	    WRITE (6,1354) IPLNS
1354	    FORMAT(' ','with ',I1,' plane per map')
	    WRITE (7,1354) IPLNS
	    IPNUM=1
	  ELSE IF (IPLNS.GT.1) THEN
	    WRITE (6,1355) IPLNS
1355	    FORMAT(' ','with ',I4,' planes per map')
	    WRITE (7,1355) IPLNS
	  END IF
	END IF
C
	IF (NCOLS.EQ.0) THEN
	  NCOLS=ICOLS
	  IF (IMNUM-NMAPS.EQ.1) THEN
	    CALL USRINP(NCOLS,1,'I',4,1,'NUMBER_OF_COLUMNS=',
	1   'Enter the number of columns for the new map:  '//
	2   '(default = number of last map)')
	  ELSE
	    CALL USRINP(NCOLS,1,'I',4,1,'NUMBER_OF_COLUMNS=',
	1   'Enter the number of columns for the new maps:  '//
	2   '(default = number of last map)')
	  END IF
	END IF
	IF (NROWS.EQ.0) THEN
	  NROWS=IROWS
	  IF (IMNUM-NMAPS.EQ.1) THEN
	    CALL USRINP(NROWS,1,'I',4,1,'NUMBER_OF_ROWS=',
	1   'Enter the number of rows for the new map:  '//
	2   '(default = number of last map)')
	  ELSE
	    CALL USRINP(NROWS,1,'I',4,1,'NUMBER_OF_ROWS=',
	1   'Enter the number of rows for the new maps:  '//
	2   '(default = number of last map)')
	  END IF
	END IF
	IF (PRGFMAT.EQ.' ') THEN
	  PRGFMAT_IN=' '
	  DO WHILE(PRGFMAT_IN.NE.'I1'.AND.PRGFMAT_IN.NE.'I2'.AND.
	1 PRGFMAT_IN.NE.'I4'.AND.PRGFMAT_IN.NE.'R4'.AND.PRGFMAT_IN.NE.
	2 'R8'.AND.PRGFMAT_IN.NE.'C8'.AND.PRGFMAT_IN.NE.'C16')
	    PRGFMAT_IN=DATFMT
	    CALL MESSAGE('Enter the data format to use:  '//
	1   '(default = format of last map')
	    CALL MESSAGE('     I1 = Unsigned 8-bit integer')
	    CALL MESSAGE('     I2 = 16-bit twos-complement binary integer')
	    CALL MESSAGE('     I4 = 32-bit twos-complement binary integer')
	    CALL MESSAGE('     R4 = IEEE single precision floating point')
	    CALL MESSAGE('     R8 = IEEE double precision floating point')
	    CALL MESSAGE('     C8 = Complex - real and imaginary are R4')
	    CALL MESSAGE('    C16 = Complex - real and imaginary are R8')
	    CALL USRINP(%REF(PRGFMAT_IN),1,'A',80,1,'DATA_FORMAT=',' ')
	    CALL ALPHA(%REF(PRGFMAT_IN),1,80,NC,NF,NR,2)
	  END DO
	  PRGFMAT=PRGFMAT_IN
	END IF
	IF (PLANEKEY.EQ.0) THEN !  Key for prompt
	  IF (PRGFMAT.EQ.'C8'.OR.PRGFMAT.EQ.'C16') THEN
	    NPLNS=2
	    CALL MESSAGE('The map(s) will contain two planes for the real '//
	1   'and imaginary parts')
	  ELSE
	    NPLNS=1
	    IF (IMNUM-NMAPS.EQ.1) THEN
	      CALL USRINP(NPLNS,1,'I',4,1,'NUMBER_OF_PLANES=',
	1     'Enter the number of planes for the new map:  (default = 1)')
	    ELSE
	      CALL USRINP(NPLNS,1,'I',4,1,'NUMBER_OF_PLANES=',
	1     'Enter the number of planes for the new maps:  (default = 1)')
	    END IF
	  END IF

	ELSE
	  NPLNS=PLANEKEY
	END IF
C  Set a default value of 1 for the plane number in the COMMON block.
	IPNUM=1
C  Prompt for the plane number if PLANEKEY=0 and more than one plane was
C  created.
	IF (PLANEKEY.EQ.0.AND.NPLNS.GT.1) THEN
	  CALL USRINP(IPNUM,1,'I',4,1,'PLANE=',
	1 'Enter the desired plane number:  (default = 1)')
	END IF
C
	NPIXELSPERMAP=NPLNS*NROWS*NCOLS
	IF (PRGFMAT.EQ.'I1') THEN
	  NBYTESPERMAP=NPIXELSPERMAP
	ELSE IF (PRGFMAT.EQ.'I2') THEN
	  NBYTESPERMAP=2*NPIXELSPERMAP
	ELSE IF (PRGFMAT.EQ.'I4'.OR.PRGFMAT.EQ.'R4'.OR.DATFMT.EQ.'C8') THEN
	  NBYTESPERMAP=4*NPIXELSPERMAP
	ELSE IF (PRGFMAT.EQ.'R8'.OR.PRGFMAT.EQ.'C16') THEN
	  NBYTESPERMAP=8*NPIXELSPERMAP
	END IF
C  Compute the number of records per map for the new maps
	NRECSPERMAP=(NBYTESPERMAP-1)/IBLKLEN+1 !  Excluding the header
	NRECSTOADD=(NRECSPERMAP+1)*(IMNUM-NMAPS) !  Including the headers of
						 !  the new maps 
C  NRECSTOADD will be revised below if the existing map has only one map.
C  We will fill the records with ASCII nulls, then fill the header(s) with
C  ASCII blanks below
	DO I=1,IBLKLEN
	  PHYSREC(I)=0
	END DO
	IRECLAST=MAPRECORDS(2,NMAPS,LU)
C
C
	IF (NMAPS.EQ.1.AND.IMNUM.GT.NMAPS) THEN
	  INCREMENT=0 !  One existing map
C  Besides adding records for additional extension header(s) and
C  images(s), add records for an extension header for existing map #1.
C  The original header now becomes the primary header and may be 
C  slightly longer in length since it must contain the EXTEND keyword.
C  Note that a preexisting EXTEND keyword does not guarantee that 
C  extensions are actually present.  The keyword BLOCKED is depracated
C  and will produce a warning if it is included in the primary header.
C  Other keywords that will produce warnings in the primary header are
C  CRPIX1, CRPIX2, etc., CRVAL1, CRVAL2, etc., CDELT1 CDELT2, etc.,
C  CTYPE1, CTYPE2, etc., CUNIT1, CUNIT2, etc.
C       The extension header for existing map #1 will be slightly
C  longer than the orginal header since it must now contain the
C  PCOUNT and GCOUNT keywords.  The SIMPLE keyword  gets replaced with
C  XTENSION keyword, and any preexisting EXTEND keyword will need to be
C  removed.
C
C  First, get the number of keycards (blank and nonblank up to and including
C  the END keyword) in the original header.
	  NKEY=NUMKEYCARDS(1,LU)
C  Compute the number of keycards modulo 36.  36 is the number of
C  keycards which can fit into a record, i.e., 2880/80
	  NKEY_36=MOD(NKEY,36)
C  Compute the number of records needed for the new primary header.
C  First find out if the EXTEND keyword exists.  If it does, then
C  the new header will be the same size as the old one.
	  LEN_PH=HEADERRECORDS(2,1,LU)-HEADERRECORDS(1,1,LU)+1
	  IF (EXTEND(LU).EQ.0) THEN !  Keyword not present
	    IF (NKEY_36.EQ.0) LEN_PH=LEN_PH+1 !  Units of records
	  END IF
C  Compute the number of records needed for the new header of map #1
	  LEN_H1=HEADERRECORDS(2,1,LU)-HEADERRECORDS(1,1,LU)+1
C  If the EXTEND keyword is present it will be removed, freeing up space for
C  an additional keyword.  We, at the same time, account for two additional
C  spaces needed for PCOUNT and GCOUNT.
	  IF (EXTEND(LU).EQ.0) THEN !  Keyword not present
	    IF (NKEY_36.EQ.35) LEN_H1=LEN_H1+1
	  ELSE
	    IF (NKEY_36.EQ.0) LEN_H1=LEN_H1+1
	  END IF
C  Make an adjustment to the number of records to add that was
C  estimated above.
	  NRECSTOADD=NRECSTOADD+LEN_PH-
	1 (HEADERRECORDS(2,1,LU)-HEADERRECORDS(1,1,LU)+1)+LEN_H1
C  Write empty records (extension header and image for each new map) to the
C  file, plus possible extra records for the primary header and map #1 header.
C  Here we write ASCII nulls.
	  DO IR=1+IRECLAST,NRECSTOADD+IRECLAST
	    WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	      STATUS=2
	    END IF
	  END DO
C  Shift the image data for map #1
	  IRSTART=MAPRECORDS(1,1,LU)
	  IREND=MAPRECORDS(2,1,LU)
	  DO IR=IREND,IRSTART,-1
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
            IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in reading physical record')
	      STATUS=2
	    END IF
	    IRWRITE=LEN_PH+LEN_H1+IR-IRSTART+1
	    WRITE(LU,REC=IRWRITE,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	      STATUS=2
	    END IF
	  END DO
C  Write the header for map #1.  We'll insert PCOUNT and GCOUNT right
C  after the NAXIS keywords
	  IRSTART=HEADERRECORDS(1,1,LU)
	  IREND=HEADERRECORDS(2,1,LU)
	  IRWRITE=HEADERRECORDS(1,1,LU)+LEN_PH
	  IBPTR_WR=1
	  INAXIS_DETECT=0
	  DONE=.FALSE.
	  CHARPHYSREC_2=' ' !  ASCII blanks
	  DO IR=IRSTART,IREND !  Loops on records for reading
	    IBPTR_RD=1
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in reading physical record')
	      STATUS=2
	    END IF
	    DO WHILE(IBPTR_RD.LE.2801.AND..NOT.DONE)
	      IF (INAXIS_DETECT.EQ.0.OR.INAXIS_DETECT.EQ.1
	1     .OR.INAXIS_DETECT.EQ.4) THEN
	        KEYCARD=CHARPHYSREC(IBPTR_RD:IBPTR_RD+79)
	      END IF
C  Replace the SIMPLE keyword with XTENSION
	      IF (KEYCARD(1:6).EQ.'SIMPLE') THEN !  The first keycard
	        KEYCARD(1:80)=' '
	        KEYCARD(1:9)='XTENSION='
	        KEYCARD(11:20)='''IMAGE   '''
	        KEYCARD(31:48)=' / Image extension'
	      ELSE IF (KEYCARD(1:5).EQ.'NAXIS') THEN
	        INAXIS_DETECT=1
	      ELSE IF (INAXIS_DETECT.EQ.1.AND.KEYCARD(1:5).NE.'NAXIS') THEN
	        INAXIS_DETECT=2
	      END IF
C  After the NAXIS keywords come PCOUNT and GCOUNT
	      IF (INAXIS_DETECT.EQ.2) THEN
	        KEYCARD(1:80)=' '
	        KEYCARD(1:9)='PCOUNT  ='
	        KEYCARD(30:30)='0'
	        KEYCARD(31:33)=' / '
	        INAXIS_DETECT=3
	      ELSE IF (INAXIS_DETECT.EQ.3) THEN
	        KEYCARD(1:80)=' '
	        KEYCARD(1:9)='GCOUNT  ='
	        KEYCARD(30:30)='1'
	        KEYCARD(31:33)=' / '
	        INAXIS_DETECT=4
	        IBPTR_RD=IBPTR_RD-80
	      END IF
	      IF (KEYCARD(1:3).EQ.'END') THEN
	        DONE=.TRUE.
	      END IF
C  If the EXTEND keycard exists, don't write it to the new header
C  Also, don't write the deprecated BLOCKED keyword.
	      IF (KEYCARD(1:6).NE.'EXTEND'.AND.
	1     KEYCARD(1:7).NE.'BLOCKED') THEN
	        CHARPHYSREC_2(IBPTR_WR:IBPTR_WR+79)=KEYCARD
	        IBPTR_WR=IBPTR_WR+80
	        IF (IBPTR_WR.EQ.2881) THEN
	          WRITE(LU,REC=IRWRITE,IOSTAT=ISTAT)
	1         (PHYSREC_2(I),I=1,IBLKLEN)
	          IF (ISTAT.NE.0) THEN
	            CALL ERROR(-3,'Error in writing physical record')
	            STATUS=2
	          END IF
	          IRWRITE=IRWRITE+1
	          IBPTR_WR=1
	          CHARPHYSREC_2=' '
	        END IF
	      END IF
	      IF (INAXIS_DETECT.EQ.0.OR.INAXIS_DETECT.EQ.1
	1     .OR.INAXIS_DETECT.EQ.4) IBPTR_RD=IBPTR_RD+80
	    END DO
	  END DO
C  Flush the buffer
	  IF (IBPTR_WR.NE.1) THEN
	    WRITE(LU,REC=IRWRITE,IOSTAT=ISTAT) (PHYSREC_2(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	      STATUS=2
	    END IF
	  END IF
C
C  Write the primary header.  We don't replace the SIMPLE keyword.  If the 
C  EXTEND keyword is not already present we will put it after the NAXIS
C  keyword.
	  IRSTART=HEADERRECORDS(1,1,LU)
	  IREND=HEADERRECORDS(2,1,LU)
	  IRWRITE=HEADERRECORDS(1,1,LU)
	  IBPTR_WR=1
	  DONE=.FALSE.
	  CHARPHYSREC_2=' ' !  ASCII blanks
	  KEYCARD_PREVIOUS=' '
	  DO IR=IRSTART,IREND !  Loops on records for reading
	    IBPTR_RD=1
	    READ(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-1,'Error in reading physical record')
	      STATUS=2
	    END IF
	    DO WHILE(IBPTR_RD.LE.2801.AND..NOT.DONE)
	      KEYCARD=CHARPHYSREC(IBPTR_RD:IBPTR_RD+79)
	      IF (KEYCARD(1:3).EQ.'END') DONE=.TRUE.
C  Put in the EXTEND keycard if it is missing and write if
C  necessary.  
	      IF (EXTEND(LU).EQ.0.AND.KEYCARD(1:5).NE.'NAXIS'.AND.
	1     KEYCARD_PREVIOUS(1:5).EQ.'NAXIS') THEN
	        KEYCARD_TEMP=' '
	        KEYCARD_TEMP(1:9)='EXTEND  ='
	        KEYCARD_TEMP(30:30)='T'
	        KEYCARD_TEMP(31:58)=' / There are FITS extensions'
	        CHARPHYSREC_2(IBPTR_WR:IBPTR_WR+79)=KEYCARD_TEMP
	        IBPTR_WR=IBPTR_WR+80
	        IF (IBPTR_WR.EQ.2881) THEN
	          WRITE(LU,REC=IRWRITE,IOSTAT=ISTAT)
	1         (PHYSREC_2(I),I=1,IBLKLEN)
	          IF (ISTAT.NE.0) THEN
	            CALL ERROR(-1,'Error in writing physical record')
	            STATUS=2
	          END IF
	          IRWRITE=IRWRITE+1
	          IBPTR_WR=1
	          CHARPHYSREC_2=' '
	        END IF
	        EXTEND(LU)=1
C  Change the keycard to true if it is false
	      ELSE IF (EXTEND(LU).EQ.2.AND.KEYCARD.EQ.'EXTEND') THEN
	        KEYCARD(30:30)='T'
	        EXTEND(LU)=1
	      END IF
	      IF (KEYCARD(1:9).EQ.'NAXIS   =') THEN
	        KEYCARD_TEMP=' '
	        KEYCARD_TEMP(1:9)='NAXIS   ='
	        KEYCARD_TEMP(30:30)='0'
	        KEYCARD_TEMP(32:32)='/'
	        CHARPHYSREC_2(IBPTR_WR:IBPTR_WR+79)=KEYCARD_TEMP
	        IBPTR_WR=IBPTR_WR+80
C  Exclude certain keycards from the primary header
	      ELSE IF (KEYCARD(1:6).NE.'NAXIS1'.AND.KEYCARD(1:6).NE.'NAXIS2'
	1     .AND.KEYCARD(1:6).NE.'NAXIS3'.AND.KEYCARD(1:7).NE.'BLOCKED'.AND.
	2     KEYCARD(1:5).NE.'CRPIX'.AND.KEYCARD(1:5).NE.'CRVAL'.AND.
	3     KEYCARD(1:5).NE.'CDELT'.AND.KEYCARD(1:5).NE.'CTYPE'.AND.
	4     KEYCARD(1:5).NE.'CUNIT') THEN
	        CHARPHYSREC_2(IBPTR_WR:IBPTR_WR+79)=KEYCARD
	        IBPTR_WR=IBPTR_WR+80
	      END IF
	      IF (IBPTR_WR.EQ.2881) THEN
	        WRITE(LU,REC=IRWRITE,IOSTAT=ISTAT)
	1       (PHYSREC_2(I),I=1,IBLKLEN)
	        IF (ISTAT.NE.0) THEN
	          CALL ERROR(-3,'Error in writing physical record')
	          STATUS=2
	        END IF
	        IRWRITE=IRWRITE+1
	        IBPTR_WR=1
	        CHARPHYSREC_2=' '
	      END IF
	      IBPTR_RD=IBPTR_RD+80
	      KEYCARD_PREVIOUS=KEYCARD
	    END DO
	  END DO
C  Flush the buffer if more keywords have been added to it
	  IF (IBPTR_WR.NE.1) THEN
	    WRITE(LU,REC=IRWRITE,IOSTAT=ISTAT) (PHYSREC_2(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	      STATUS=2
	    END IF
	  END IF
C  In order to write new keywords, there has to be some information in the
C  common blocks about the the headers, maps, and records.
	  NUMRECORDS(LU)=NRECSTOADD+IRECLAST
	  DO N=NMAPS+1,IMNUM
	    NUMKEYCARDS(N,LU)=0
	  END DO
	  NUMMAPS(LU)=IMNUM
C  Update the header record data
C  For the new map headers we assume just one record per header.
C  Note that NMAPS=1 here.
	  HEADERRECORDS(2,0,LU)=LEN_PH
	  HEADERRECORDS(1,1,LU)=LEN_PH+1
	  HEADERRECORDS(2,1,LU)=HEADERRECORDS(1,1,LU)+LEN_H1-1
	  HEADERRECORDS(1,NMAPS+1,LU)=HEADERRECORDS(2,1,LU)+
	1 MAPRECORDS(2,1,LU)-MAPRECORDS(1,1,LU)+2
	  HEADERRECORDS(2,NMAPS+1,LU)=HEADERRECORDS(1,NMAPS+1,LU)
	  CHARPHYSREC_2=' '
	  WRITE (LU,REC=HEADERRECORDS(1,NMAPS+1,LU),IOSTAT=ISTAT)
	1 (PHYSREC_2(I),I=1,IBLKLEN)
	  IF (ISTAT.NE.0) THEN
	    CALL ERROR(-3,'Error in writing physical record')
	    STATUS=2
	  END IF
C
	  I=1
	  DO N=NMAPS+2,IMNUM
	    HEADERRECORDS(1,N,LU)=HEADERRECORDS(2,NMAPS+1,LU)+
	1   (NRECSPERMAP+1)*I
	    HEADERRECORDS(2,N,LU)=HEADERRECORDS(1,N,LU)
	    WRITE (LU,REC=HEADERRECORDS(1,N,LU),IOSTAT=ISTAT)
	1   (PHYSREC_2(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-2,'Error in writing physical record')
	      STATUS=2
	    END IF
	    I=I+1
	  END DO
C  Now we are ready to write keycards for the new maps
	  DO M=NMAPS+1,IMNUM
	    IF (M.NE.0) THEN
	      CALL WRFITSMHST(LU,M,'XTENSION','''IMAGE   ''','C',
	1     'Image extension',.TRUE.)
	    END IF
	    IF (PRGFMAT.EQ.'I1') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',8.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'I2') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',16.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'I4') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',32.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'R4'.OR.PRGFMAT.EQ.'C8') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',-32.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'R8'.OR.PRGFMAT.EQ.'C16') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',-64.0D0,'I4',' ',.TRUE.)
	    END IF
	    CALL WRFITSMHV(LU,M,'NAXIS',3.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS1',NCOLS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS2',NROWS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS3',NPLNS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'PCOUNT',0.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'GCOUNT',1.0D0,'I4',' ',.TRUE.)
	    IF (PRGFMAT.EQ.'C8'.OR.PRGFMAT.EQ.'C16')
	1   CALL WRFITSMHST(LU,M,'CTYPE3','''COMPLEX ''','C',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'END',0.0D0,'I4',' ',.TRUE.)
	  END DO
C  Update the common block information
	  CALL GET_FITS_INFO(LU)
C  Update NMAPS
	  NMAPS=NUMMAPS(LU)
C
C
	ELSE IF (NMAPS.GT.1.AND.IMNUM.GT.NMAPS) THEN
C  Write ASCII nulls to the records for the extension header and image
C  for each new map, then write ASCII blanks to the header below.  The
C  array PHYSREC was loaded with nulls earlier.
	  DO IR=1+IRECLAST,NRECSTOADD+IRECLAST
	    WRITE(LU,REC=IR,IOSTAT=ISTAT) (PHYSREC(I),I=1,IBLKLEN)
	    IF (ISTAT.NE.0) THEN
	      CALL ERROR(-3,'Error in writing physical record')
	      STATUS=2
	    END IF
	  END DO
C  Before we write new keywords, we need to update the information in the
C  common blocks about the the headers, maps, and records.
	  NUMRECORDS(LU)=NRECSTOADD+IRECLAST
	  DO N=NMAPS+1,IMNUM
	    NUMKEYCARDS(N,LU)=0
	  END DO
	  NUMMAPS(LU)=IMNUM
C  ASCII blanks now
	  DO I=1,IBLKLEN
	    PHYSREC(I)=32
	  END DO
C  Update the new map header records.  We only need one record per header.
	  II=1
	  DO N=NMAPS+1,IMNUM
	    IF (N.EQ.NMAPS+1) THEN
	      HEADERRECORDS(1,N,LU)=IRECLAST+1
	    ELSE
	      HEADERRECORDS(1,N,LU)=HEADERRECORDS(2,NMAPS,LU)+
	1     (NRECSPERMAP+1)*II
	    END IF
	    HEADERRECORDS(2,N,LU)=HEADERRECORDS(1,N,LU)
	    II=II+1
	    WRITE(LU,REC=HEADERRECORDS(1,N,LU),IOSTAT=ISTAT)
	1   (PHYSREC(I),I=1,IBLKLEN)
	  END DO
C  Now we are ready to write keycards
	  DO M=NMAPS+1,IMNUM
	    IF (NMAPS.GT.1.AND.M.NE.0) THEN
	      CALL WRFITSMHST(LU,M,'XTENSION','''IMAGE   ''','C',
	1     'Image extension',.TRUE.)
	    END IF
	    IF (PRGFMAT.EQ.'I1') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',8.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'I2') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',16.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'I4') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',32.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'R4'.OR.PRGFMAT.EQ.'C8') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',-32.0D0,'I4',' ',.TRUE.)
	    ELSE IF (PRGFMAT.EQ.'R8'.OR.PRGFMAT.EQ.'C16') THEN
	      CALL WRFITSMHV(LU,M,'BITPIX',-64.0D0,'I4',' ',.TRUE.)
	    END IF
	    CALL WRFITSMHV(LU,M,'NAXIS',3.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS1',NCOLS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS2',NROWS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'NAXIS3',NPLNS*1.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'PCOUNT',0.0D0,'I4',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'GCOUNT',1.0D0,'I4',' ',.TRUE.)
	    IF (PRGFMAT.EQ.'C8'.OR.PRGFMAT.EQ.'C16')
	1   CALL WRFITSMHST(LU,M,'CTYPE3','''COMPLEX ''','C',' ',.TRUE.)
	    CALL WRFITSMHV(LU,M,'END',0.0D0,'I4',' ',.TRUE.)
	  END DO
C  Update the common block information
	  CALL GET_FITS_INFO(LU)
	  NMAPS=NUMMAPS(LU)
	END IF
	RETURN
	END
